=== modified file 'drizzled/create_field.cc'
--- drizzled/create_field.cc	2009-06-13 21:21:13 +0000
+++ drizzled/create_field.cc	2009-06-30 06:09:40 +0000
@@ -42,6 +42,7 @@
 #include "drizzled/field/timestamp.h"
 #include "drizzled/field/datetime.h"
 #include "drizzled/field/varstring.h"
+#include "drizzled/temporal.h"
 
 /** Create a field suitable for create of table. */
 CreateField::CreateField(Field *old_field, Field *orig_field)
@@ -290,18 +291,15 @@
     case DRIZZLE_TYPE_TIMESTAMP:
       if (!fld_length)
       {
-        /* Compressed date YYYYMMDDHHMMSS */
-        length= MAX_DATETIME_COMPRESSED_WIDTH;
-      }
-      else if (length != MAX_DATETIME_WIDTH)
-      {
-        /*
-          We support only even TIMESTAMP lengths less or equal than 14
-          and 19 as length of 4.1 compatible representation.
-        */
-        length= ((length+1)/2)*2; /* purecov: inspected */
-        length= cmin(length, (uint32_t)MAX_DATETIME_COMPRESSED_WIDTH); /* purecov: inspected */
-      }
+        length= drizzled::DateTime::MAX_STRING_LENGTH;
+      }
+
+      /* This assert() should be correct due to absence of length
+         specifiers for timestamp. Previous manipulation also wasn't
+         ever called (from examining lcov)
+      */
+      assert(length == (uint32_t)drizzled::DateTime::MAX_STRING_LENGTH);
+
       flags|= UNSIGNED_FLAG;
       if (fld_default_value)
       {
@@ -342,10 +340,10 @@
       }
       break;
     case DRIZZLE_TYPE_DATE:
-      length= 10;
+      length= drizzled::Date::MAX_STRING_LENGTH;
       break;
     case DRIZZLE_TYPE_DATETIME:
-      length= MAX_DATETIME_WIDTH;
+      length= drizzled::DateTime::MAX_STRING_LENGTH;
       break;
     case DRIZZLE_TYPE_ENUM:
       {

=== modified file 'drizzled/definitions.h'
--- drizzled/definitions.h	2009-06-27 18:59:16 +0000
+++ drizzled/definitions.h	2009-06-30 06:09:40 +0000
@@ -62,10 +62,6 @@
 /* Max column width +1 */
 #define MAX_FIELD_WIDTH		(MAX_FIELD_CHARLENGTH*MAX_MBWIDTH+1)
 
-#define MAX_DATE_WIDTH		10	/* YYYY-MM-DD */
-#define MAX_TIME_WIDTH		23	/* -DDDDDD HH:MM:SS.###### */
-#define MAX_DATETIME_FULL_WIDTH 29	/* YYYY-MM-DD HH:MM:SS.###### AM */
-#define MAX_DATETIME_WIDTH	19	/* YYYY-MM-DD HH:MM:SS */
 #define MAX_DATETIME_COMPRESSED_WIDTH 14  /* YYYYMMDDHHMMSS */
 
 #define MAX_TABLES	(sizeof(table_map)*8-3)	/* Max tables in join */

=== modified file 'drizzled/field/datetime.cc'
--- drizzled/field/datetime.cc	2009-06-29 06:38:07 +0000
+++ drizzled/field/datetime.cc	2009-06-30 06:09:40 +0000
@@ -175,12 +175,9 @@
 String *Field_datetime::val_str(String *val_buffer,
 				String *)
 {
-  val_buffer->alloc(field_length);
-  val_buffer->length(field_length);
-  uint64_t tmp;
-  long part1,part2;
-  char *pos;
-  int part3;
+  val_buffer->alloc(drizzled::DateTime::MAX_STRING_LENGTH);
+  val_buffer->length(drizzled::DateTime::MAX_STRING_LENGTH);
+  int64_t tmp;
 
 #ifdef WORDS_BIGENDIAN
   if (table && table->s->db_low_byte_first)
@@ -189,34 +186,25 @@
 #endif
     int64_tget(tmp,ptr);
 
-  /*
-    Avoid problem with slow int64_t arithmetic and sprintf
-  */
-
-  part1=(long) (tmp/INT64_C(1000000));
-  part2=(long) (tmp - (uint64_t) part1*INT64_C(1000000));
-
-  pos=(char*) val_buffer->ptr() + MAX_DATETIME_WIDTH;
-  *pos--=0;
-  *pos--= (char) ('0'+(char) (part2%10)); part2/=10;
-  *pos--= (char) ('0'+(char) (part2%10)); part3= (int) (part2 / 10);
-  *pos--= ':';
-  *pos--= (char) ('0'+(char) (part3%10)); part3/=10;
-  *pos--= (char) ('0'+(char) (part3%10)); part3/=10;
-  *pos--= ':';
-  *pos--= (char) ('0'+(char) (part3%10)); part3/=10;
-  *pos--= (char) ('0'+(char) part3);
-  *pos--= ' ';
-  *pos--= (char) ('0'+(char) (part1%10)); part1/=10;
-  *pos--= (char) ('0'+(char) (part1%10)); part1/=10;
-  *pos--= '-';
-  *pos--= (char) ('0'+(char) (part1%10)); part1/=10;
-  *pos--= (char) ('0'+(char) (part1%10)); part3= (int) (part1/10);
-  *pos--= '-';
-  *pos--= (char) ('0'+(char) (part3%10)); part3/=10;
-  *pos--= (char) ('0'+(char) (part3%10)); part3/=10;
-  *pos--= (char) ('0'+(char) (part3%10)); part3/=10;
-  *pos=(char) ('0'+(char) part3);
+  drizzled::DateTime dt;
+
+  /* TODO: add an assert that this succeeds
+   * currently fails due to bug in allowing
+   * ALTER TABLE to add a datetime column that's
+   * not null without a default value.
+   */
+  dt.from_int64_t(tmp, false); /* NOTE: this does *NOT* attempt convertion
+				        from formats such as 20090101 as
+					the stored value has already been
+					converted.
+			       */
+
+  int rlen;
+  rlen= dt.to_string((char*)val_buffer->ptr(), drizzled::DateTime::MAX_STRING_LENGTH);
+  assert((rlen+1) <  drizzled::DateTime::MAX_STRING_LENGTH);
+
+  val_buffer->length(rlen);
+
   return val_buffer;
 }
 

=== modified file 'drizzled/field/timestamp.cc'
--- drizzled/field/timestamp.cc	2009-06-29 06:38:07 +0000
+++ drizzled/field/timestamp.cc	2009-06-30 06:09:40 +0000
@@ -79,7 +79,8 @@
                                  const char *field_name_arg,
                                  TableShare *share,
                                  const CHARSET_INFO * const cs)
-  :Field_str(ptr_arg, MAX_DATETIME_WIDTH, null_ptr_arg, null_bit_arg,
+  :Field_str(ptr_arg, drizzled::DateTime::MAX_STRING_LENGTH,
+             null_ptr_arg, null_bit_arg,
 	     unireg_check_arg, field_name_arg, cs)
 {
   /* For 4.0 MYD and 4.0 InnoDB compatibility */
@@ -97,7 +98,7 @@
 Field_timestamp::Field_timestamp(bool maybe_null_arg,
                                  const char *field_name_arg,
                                  const CHARSET_INFO * const cs)
-  :Field_str((unsigned char*) 0, MAX_DATETIME_WIDTH,
+  :Field_str((unsigned char*) 0, drizzled::DateTime::MAX_STRING_LENGTH,
              maybe_null_arg ? (unsigned char*) "": 0, 0,
 	     NONE, field_name_arg, cs)
 {
@@ -251,7 +252,7 @@
   rlen= temporal.to_string(to, to_len);
   assert(rlen < to_len);
 
-  val_buffer->length(to_len);
+  val_buffer->length(rlen);
   return val_buffer;
 }
 

=== modified file 'drizzled/function/time/curdate.cc'
--- drizzled/function/time/curdate.cc	2009-03-01 03:08:20 +0000
+++ drizzled/function/time/curdate.cc	2009-06-30 06:09:40 +0000
@@ -28,7 +28,7 @@
 {
   collation.set(&my_charset_bin);
   decimals=0;
-  max_length=MAX_DATE_WIDTH*MY_CHARSET_BIN_MB_MAXLEN;
+  max_length=drizzled::Date::MAX_STRING_LENGTH*MY_CHARSET_BIN_MB_MAXLEN;
 
   store_now_in_TIME(&ltime);
 

=== modified file 'drizzled/function/time/date.h'
--- drizzled/function/time/date.h	2009-02-14 19:02:06 +0000
+++ drizzled/function/time/date.h	2009-06-30 06:09:40 +0000
@@ -22,9 +22,7 @@
 
 #include "drizzled/function/func.h"
 #include "drizzled/function/str/strfunc.h"
-
-/* forward declarations */
-namespace drizzled {class Date;}
+#include <drizzled/temporal.h>
 
 /* A function which evaluates to a Date */
 class Item_date :public Item_func
@@ -44,7 +42,7 @@
   {
     collation.set(&my_charset_bin);
     decimals=0;
-    max_length=MAX_DATE_WIDTH*MY_CHARSET_BIN_MB_MAXLEN;
+    max_length=drizzled::Date::MAX_STRING_LENGTH*MY_CHARSET_BIN_MB_MAXLEN;
   }
   /**
    * All functions which inherit from Item_date must implement

=== modified file 'drizzled/function/time/date_add_interval.cc'
--- drizzled/function/time/date_add_interval.cc	2009-02-28 17:49:44 +0000
+++ drizzled/function/time/date_add_interval.cc	2009-06-30 06:09:40 +0000
@@ -44,7 +44,7 @@
 
   collation.set(&my_charset_bin);
   maybe_null=1;
-  max_length=MAX_DATETIME_FULL_WIDTH*MY_CHARSET_BIN_MB_MAXLEN;
+  max_length=drizzled::DateTime::MAX_STRING_LENGTH*MY_CHARSET_BIN_MB_MAXLEN;
   value.alloc(max_length);
 
   /*
@@ -104,15 +104,16 @@
   else if (ltime.second_part)
   {
     /* Ensure we've got enough room for our timestamp string. */
-    str->length(MAX_DATETIME_FULL_WIDTH);
-    size_t length= sprintf(str->c_ptr(), "%04u-%02u-%02u %02u:%02u:%02u.%06u",
-                          ltime.year,
-                          ltime.month,
-                          ltime.day,
-                          ltime.hour,
-                          ltime.minute,
-                          ltime.second,
-                          (uint32_t) ltime.second_part);
+    str->length(drizzled::DateTime::MAX_STRING_LENGTH);
+    size_t length= snprintf(str->c_ptr(), drizzled::DateTime::MAX_STRING_LENGTH,
+                            "%04u-%02u-%02u %02u:%02u:%02u.%06u",
+                            ltime.year,
+                            ltime.month,
+                            ltime.day,
+                            ltime.hour,
+                            ltime.minute,
+                            ltime.second,
+                            (uint32_t) ltime.second_part);
     str->length(length);
     str->set_charset(&my_charset_bin);
   }

=== modified file 'drizzled/function/time/from_unixtime.cc'
--- drizzled/function/time/from_unixtime.cc	2009-02-28 20:43:31 +0000
+++ drizzled/function/time/from_unixtime.cc	2009-06-30 06:09:40 +0000
@@ -32,7 +32,7 @@
   session= current_session;
   collation.set(&my_charset_bin);
   decimals= DATETIME_DEC;
-  max_length=MAX_DATETIME_WIDTH*MY_CHARSET_BIN_MB_MAXLEN;
+  max_length=drizzled::DateTime::MAX_STRING_LENGTH*MY_CHARSET_BIN_MB_MAXLEN;
   maybe_null= 1;
 }
 

=== modified file 'drizzled/function/time/makedate.h'
--- drizzled/function/time/makedate.h	2008-12-12 20:40:37 +0000
+++ drizzled/function/time/makedate.h	2009-06-30 06:09:40 +0000
@@ -32,7 +32,7 @@
   void fix_length_and_dec()
   {
     decimals=0;
-    max_length=MAX_DATE_WIDTH*MY_CHARSET_BIN_MB_MAXLEN;
+    max_length=drizzled::Date::MAX_STRING_LENGTH*MY_CHARSET_BIN_MB_MAXLEN;
   }
   int64_t val_int();
 };

=== modified file 'drizzled/function/time/sysdate_local.cc'
--- drizzled/function/time/sysdate_local.cc	2009-01-23 06:03:42 +0000
+++ drizzled/function/time/sysdate_local.cc	2009-06-30 06:09:40 +0000
@@ -63,7 +63,7 @@
 {
   decimals= 0;
   collation.set(&my_charset_bin);
-  max_length= MAX_DATETIME_WIDTH*MY_CHARSET_BIN_MB_MAXLEN;
+  max_length= drizzled::DateTime::MAX_STRING_LENGTH*MY_CHARSET_BIN_MB_MAXLEN;
 }
 
 

=== modified file 'drizzled/function/time/typecast.h'
--- drizzled/function/time/typecast.h	2009-02-21 16:00:06 +0000
+++ drizzled/function/time/typecast.h	2009-06-30 06:09:40 +0000
@@ -21,6 +21,7 @@
 #define DRIZZLED_FUNCTION_TIME_TYPECAST_H
 
 #include <drizzled/function/str/strfunc.h>
+#include <drizzled/temporal.h>
 
 class Item_typecast :public Item_str_func
 {
@@ -133,7 +134,7 @@
   {
     collation.set(&my_charset_bin);
     maybe_null= 1;
-    max_length= MAX_DATETIME_FULL_WIDTH * MY_CHARSET_BIN_MB_MAXLEN;
+    max_length= drizzled::DateTime::MAX_STRING_LENGTH * MY_CHARSET_BIN_MB_MAXLEN;
     decimals= DATETIME_DEC;
   }
   bool result_as_int64_t() { return true; }

=== modified file 'drizzled/function/time/unix_timestamp.cc'
--- drizzled/function/time/unix_timestamp.cc	2009-06-29 06:38:07 +0000
+++ drizzled/function/time/unix_timestamp.cc	2009-06-30 06:09:40 +0000
@@ -63,11 +63,10 @@
   if (! temporal.is_valid())
   {
     null_value= true;
-    char buff[MAX_DATETIME_WIDTH];
+    char buff[drizzled::DateTime::MAX_STRING_LENGTH];
     int buff_len;
-    buff_len= temporal.to_string(buff, MAX_DATETIME_WIDTH);
-    assert(buff_len < MAX_DATETIME_WIDTH);
-    buff[buff_len]= '\0';
+    buff_len= temporal.to_string(buff, drizzled::DateTime::MAX_STRING_LENGTH);
+    assert((buff_len+1) < drizzled::DateTime::MAX_STRING_LENGTH);
     my_error(ER_INVALID_UNIX_TIMESTAMP_VALUE, MYF(0), buff);
     return 0;
   }

=== modified file 'drizzled/opt_range.cc'
--- drizzled/opt_range.cc	2009-06-29 06:38:07 +0000
+++ drizzled/opt_range.cc	2009-06-30 06:09:40 +0000
@@ -4554,7 +4554,7 @@
     if (value->real_item()->type() == Item::FIELD_ITEM
         && value->result_type() == STRING_RESULT)
     {
-      char buff[MAX_DATETIME_FULL_WIDTH];
+      char buff[drizzled::DateTime::MAX_STRING_LENGTH];
       String tmp(buff, sizeof(buff), &my_charset_bin);
       String *res= value->val_str(&tmp);
 
@@ -4584,13 +4584,13 @@
            * Datetime in right-hand side column is before UNIX epoch, so adjust to
            * lower bound.
            */
-          char new_value_buff[MAX_DATETIME_FULL_WIDTH];
-          size_t new_value_length;
+          char new_value_buff[drizzled::DateTime::MAX_STRING_LENGTH];
+          int new_value_length;
           String new_value_string(new_value_buff, sizeof(new_value_buff), &my_charset_bin);
 
           new_value_length= min_timestamp.to_string(new_value_string.c_ptr(),
-						    MAX_DATETIME_FULL_WIDTH);
-	  assert(new_value_length < MAX_DATETIME_FULL_WIDTH);
+				    drizzled::DateTime::MAX_STRING_LENGTH);
+	  assert((new_value_length+1) < drizzled::DateTime::MAX_STRING_LENGTH);
           new_value_string.length(new_value_length);
           err= value->save_str_value_in_field(field, &new_value_string);
         }
@@ -4600,13 +4600,13 @@
            * Datetime in right hand side column is after UNIX epoch, so adjust
            * to the higher bound of the epoch.
            */
-          char new_value_buff[MAX_DATETIME_FULL_WIDTH];
+          char new_value_buff[drizzled::DateTime::MAX_STRING_LENGTH];
           int new_value_length;
           String new_value_string(new_value_buff, sizeof(new_value_buff), &my_charset_bin);
 
           new_value_length= max_timestamp.to_string(new_value_string.c_ptr(),
-						    MAX_DATETIME_FULL_WIDTH);
-	  assert(new_value_length < MAX_DATETIME_FULL_WIDTH);
+					drizzled::DateTime::MAX_STRING_LENGTH);
+	  assert((new_value_length+1) < drizzled::DateTime::MAX_STRING_LENGTH);
           new_value_string.length(new_value_length);
           err= value->save_str_value_in_field(field, &new_value_string);
         }

=== modified file 'drizzled/temporal.cc'
--- drizzled/temporal.cc	2009-06-29 06:38:07 +0000
+++ drizzled/temporal.cc	2009-06-30 06:09:40 +0000
@@ -1349,7 +1349,7 @@
  * This is pretty much a hack for usability, but keeps us compatible
  * with MySQL.
  */
-bool DateTime::from_int64_t(const int64_t from)
+bool DateTime::from_int64_t(const int64_t from, bool convert)
 {
   int64_t copy_from= from;
   int64_t part1;
@@ -1358,7 +1358,7 @@
   if (copy_from == 0LL)
     return false;
 
-  if (copy_from < 10000101000000LL)
+  if (convert && copy_from < 10000101000000LL)
   {
     if (copy_from < 101)
       return false;

=== modified file 'drizzled/temporal.h'
--- drizzled/temporal.h	2009-06-29 06:38:07 +0000
+++ drizzled/temporal.h	2009-06-30 06:09:40 +0000
@@ -339,6 +339,12 @@
   virtual int to_string(char *to, size_t to_len) const;
 
   /**
+   * Maximum length of C-String needed to represent type
+   * (including '\0').
+   */
+  static const int MAX_STRING_LENGTH= 11;
+
+  /**
    * Attempts to populate the Date instance based
    * on the contents of a supplied string.
    *
@@ -515,6 +521,13 @@
   int to_string(char *to, size_t to_len) const;
 
   /**
+   * Maximum length of C-String needed to represent type
+   * (including '\0').
+   */
+  static const int MAX_STRING_LENGTH= 9;
+
+
+  /**
    * Attempts to populate the Time instance based
    * on the contents of a supplied string.
    *
@@ -613,6 +626,12 @@
   virtual int to_string(char *to, size_t to_len) const;
 
   /**
+   * Maximum length of C-String needed to represent type
+   * (including '\0').
+   */
+  static const int MAX_STRING_LENGTH= 27;
+
+  /**
    * Attempts to populate the DateTime instance based
    * on the contents of a supplied string.
    *
@@ -652,8 +671,14 @@
    * successful.
    *
    * @param Integer to convert from
+   * @param convert if conversion to canonical representation
+   *        should be attempted
    */
-  bool from_int64_t(const int64_t from);
+  bool from_int64_t(const int64_t from, bool convert);
+
+  bool from_int64_t(const int64_t from) {
+    return from_int64_t(from, true);
+  }
 
   /**
    * Fills a supplied tm pointer with an
@@ -765,6 +790,12 @@
   int to_string(char *to, size_t to_len) const;
 
   /**
+   * Maximum length of C-String needed to represent type
+   * (including '\0').
+   */
+  static const int MAX_STRING_LENGTH= 27;
+
+  /**
    * Fills a supplied timeval pointer with an
    * representation of the MicroTimestamp
    * value.

